#pragma warning (disable : 4305)
#pragma warning (disable : 4244)
#pragma warning (disable : 4786)
#pragma warning (disable : 4312)

#include "modelerview.h"
#include "modelerdraw.h"
#include "modelerui.h"
#include "vault.h"
#include "FL/glut.h"
#include "model.h"
#include <cmath>
#include <time.h>

class DustBunny : public Model
{

protected:
	
	void setUpLowerLeg();
	void drawLowerLeftLeg();
	void drawLowerRightLeg();

	void setUpUpperLeg();
	void drawUpperLeftLeg();
	void drawUpperRightLeg();

	void setUpBody();
	void drawBody();

	void drawUpperRightArm();
	void drawLowerRightArm();

	void drawUpperLeftArm();
	void drawLowerLeftArm();

	void drawRightEye();
	void drawLeftEye();
	void drawPupil();


	GLfloat* furCoords;
	GLuint* furDrawOrder;
	void drawFur();

public:

	// Knee joint
	RangeProperty leftLowerLeg;
	RangeProperty rightLowerLeg;

	// Hip bone
	RangeProperty rightUpperLegAlpha;
	RangeProperty rightUpperLegBeta;
	RangeProperty leftUpperLegAlpha;
	RangeProperty leftUpperLegBeta;

	// Shoulder rotation
	RangeProperty leftShoulder;
	RangeProperty rightShoulder;

	// Part of shoulder joint
	RangeProperty leftUpperArmAlpha;
	RangeProperty leftUpperArmBeta;
	RangeProperty rightUpperArmAlpha;
	RangeProperty rightUpperArmBeta;

	// Elbow joint
	RangeProperty leftLowerArm;
	RangeProperty rightLowerArm;

	// Body position
	RangeProperty bodyX;
	RangeProperty bodyY;
	RangeProperty bodyZ;

	// Body rotation
	RangeProperty bodyAlpha;
	RangeProperty bodyBeta;
	RangeProperty bodyGamma;

	// Eye rotation
	RangeProperty eyeAlpha;
	RangeProperty eyeBeta;

	Texture2D whiteTexture;
	Texture2D blackTexture;

	
	/** Construct the scene */
	DustBunny() :
		// You have to call the parent class's constructor, to provide a
		// name for the model.
		Model("DustBunny"),
		whiteTexture("white.png"),
		blackTexture("black.png"),
		leftLowerLeg("Flex Left Leg", 0, 180, 0, 1),
		rightLowerLeg("Flex Right Leg", 0, 180, 0, 1),

		rightUpperLegAlpha("Upper right leg (front to back)", -90, 90, 0, 1),
		rightUpperLegBeta("Upper right leg (sides)", -90, 90, 0, 1),
		leftUpperLegAlpha("Upper left leg (front to back)", -90, 90, 0, 1),
		leftUpperLegBeta("Upper left leg (sides)", -90, 90, 0, 1),

		leftShoulder("Rotate left shoulder", -180, 180, 90, 1),
		rightShoulder("Rotate right shoulder", -180, 180, 90, 1),

		leftUpperArmAlpha("Left upper arm (up down)", -120, 120, 0, 1),
		leftUpperArmBeta("Left upper arm (front back)", -90, 90, 0, 1),
		rightUpperArmAlpha ("Right upper arm (up down)", -120, 120, 0, 1),
		rightUpperArmBeta("Right upper arm (front back)", -90, 90, 0, 1),

		leftLowerArm("Flex left arm", 0, 180, 0, 1),
		rightLowerArm("Flex right arm", 0, 180, 0, 1),

		bodyX("Body X", -10.0f, 10.0f, 0.0f, 0.1f),
		bodyY("Body Y", -10.0f, 10.0f, 2.5f, 0.1f),
		bodyZ("Body Z", -10.0f, 10.0f, 0.0f, 0.1f),

		bodyAlpha("Body Pitch", -180, 180, 0, 1),
		bodyBeta("Body Yaw", -180, 180, 0, 1),
		bodyGamma("Body Roll", -180, 180, 0, 1),

		eyeAlpha("Eye (up down)", -90, 90, 0, 1),
		eyeBeta("Eye (left right)", -90, 90, 0, 1)
	{
		properties.add(&bodyX)
			.add(&bodyY)
			.add(&bodyZ)
			.add(&bodyAlpha)
			.add(&bodyBeta)
			.add(&bodyGamma)
			.add(&eyeAlpha)
			.add(&eyeBeta)
			.add(&rightUpperLegAlpha)
			.add(&rightUpperLegBeta)
			.add(&leftUpperLegAlpha)
			.add(&leftUpperLegBeta)
			.add(&leftLowerLeg)
			.add(&rightLowerLeg)
			.add(&rightShoulder)
			.add(&rightUpperArmAlpha)
			.add(&rightUpperArmBeta)
			.add(&rightLowerArm)
			.add(&leftShoulder)
			.add(&leftUpperArmAlpha)
			.add(&leftUpperArmBeta)
			.add(&leftLowerArm);
	}

	void load() {
		whiteTexture.load();
		blackTexture.load();
	}

	void draw();
};

